@@ -1,6 +1,7 @@
 ###           groupedData - data frame with a grouping structure
 ###
-### Copyright 1997-2003  Jose C. Pinheiro <Jose.Pinheiro@pharma.novartis.com>,
+### Copyright 2006-2017  The R Core team
+### Copyright 1997-2003  Jose C. Pinheiro,
 ###                      Douglas M. Bates <bates@stat.wisc.edu>
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -28,14 +29,15 @@ groupedData <-
            inner = NULL, labels = NULL, units = NULL)
 {
   if (!(inherits(formula, "formula") && length(formula) == 3)) {
-    stop("first argument to groupedData must be a two-sided formula")
+    stop("first argument to 'groupedData' must be a two-sided formula")
   }
   if (is.null(grpForm <- getGroupsFormula(formula, asList = TRUE))) {
-    stop("Right hand side of first argument must be a conditional expression")
+    stop("right-hand side of first argument must be a conditional expression")
   }
   mCall <- match.call()
-  mCall[[1]] <- as.name(ifelse(length(grpForm) == 1, "nfGroupedData",
-                               "nmGroupedData"))
+  mCall[[1]] <- if(length(grpForm) == 1)
+		     quote(nlme::nfGroupedData)
+		else quote(nlme::nmGroupedData)
   eval(mCall, envir = parent.frame())
 }
 
@@ -48,14 +50,14 @@ nfGroupedData <-
            inner = NULL, labels = NULL, units = NULL)
 {
   if (!(inherits(formula, "formula") && length(formula) == 3)) {
-    stop("first argument to nfGroupedData must be a two-sided formula")
+    stop("first argument to 'nfGroupedData' must be a two-sided formula")
   }
   grpForm <- getGroupsFormula(formula, asList = TRUE)
   if (is.null(grpForm)) {
-    stop("Right hand side of first argument must be a conditional expression")
+    stop("right-hand side of first argument must be a conditional expression")
   }
   if (length(grpForm) > 1) {
-    stop("Only one level of grouping allowed")
+    stop("only one level of grouping allowed")
   }
   ## create a data frame in which formula, inner, and outer can be evaluated
   if (missing(data)) {
@@ -65,7 +67,7 @@ nfGroupedData <-
     data <- do.call('data.frame', alist)
   } else {
     if (!inherits(data, "data.frame")) {
-      stop("second argument to groupedData must inherit from data.frame")
+      stop("second argument to 'groupedData' must inherit from data.frame")
     }
   }
   ## Although response and primary are not always used, they are
@@ -134,7 +136,7 @@ nmGroupedData <-
     if (is.null(object)) return(object)
     if (is.list(object)) {
       if (is.null(names(object))) {
-        names(object) <- nams[1:length(object)]
+        names(object) <- nams[seq_along(object)]
       }
       return(object)
     }
@@ -148,11 +150,11 @@ nmGroupedData <-
     object
   }
   if (!(inherits(formula, "formula") && length(formula) == 3)) {
-    stop("first argument to nmGroupedData must be a two-sided formula")
+    stop("first argument to 'nmGroupedData' must be a two-sided formula")
   }
   grpForm <- getGroupsFormula(formula, asList = TRUE)
   if (is.null(grpForm)) {
-    stop("Right hand side of first argument must be a conditional expression")
+    stop("right-hand side of first argument must be a conditional expression")
   }
   if (length(grpForm) == 1) {           # single group
     mCall <- match.call()[-1]
@@ -174,7 +176,7 @@ nmGroupedData <-
     data <- do.call('data.frame', alist)
   } else {
     if (!inherits(data, "data.frame")) {
-      stop("second argument to groupedData must inherit from data.frame")
+      stop("second argument to 'groupedData' must inherit from data.frame")
     }
   }
   ## Although response and primary are not always used, they are
@@ -217,7 +219,7 @@ collapse.groupedData <-
   Q <- length(grpForm)                  # number of levels
   if (Q == 1) {                         # no collapsing
     if (!missing(subset)) {
-      warning("Subset ignored with single grouping factor")
+      warning("'subset' ignored with single grouping factor")
     }
     return(object)
   }
@@ -225,14 +227,14 @@ collapse.groupedData <-
   if (!is.null(subset)) {
     ## choosing some levels of grouping factors
     if (!is.list(subset)) {
-      stop("\"Subset\" must be a list")
+      stop("'subset' must be a list")
     }
     if (!any(is.na(match(names(subset), 1:Q)))) {
       ## subset names given as integers
       names(subset) <- grpNames[names(subset)]
     }
     if (any(is.na(match(names(subset), grpNames)))) {
-      stop("Undefined group declared in \"subset\"")
+      stop("undefined group declared in 'subset'")
     }
     auxSubset <- rep(TRUE, dim(object)[1])
     for(i in names(subset)) {
@@ -243,11 +245,11 @@ collapse.groupedData <-
     groups[] <- lapply(groups, function(x) x[drop = TRUE])
   }
   if (length(displayLevel) != 1) {
-    stop("Only one display level allowed")
+    stop("only one display level allowed")
   }
   if (is.null(grpForm[[displayLevel]])) {
-    stop(paste("Undefined display level",displayLevel,"for",
-	       substitute(object)))
+      stop(gettextf("undefined display level %s for %s",
+                    displayLevel, sQuote(substitute(object))), domain = NA)
   }
   attribs <- attributes(object)
   ord <- attribs[["order.groups"]][[displayLevel]]
@@ -272,8 +274,8 @@ collapse.groupedData <-
 	    }
   if (dlevel < Q) {			# may need to collapse object
     if (is.null(grpForm[[collapseLevel]])) {
-      stop(paste("Undefined collapsing level", collapseLevel,
-		 "for", substitute(object)))
+        stop(gettextf("undefined collapsing level %s for %s",
+                      collapseLevel, sQuote(substitute(object))), domain = NA)
     }
     clevel <- if (is.character(collapseLevel)) {
       match(collapseLevel, grpNames)
@@ -282,8 +284,7 @@ collapse.groupedData <-
     }
     if (clevel < dlevel) {
       clevel <- dlevel
-      warning(paste("Collapsing level cannot be smaller than display level;",
-		    "setting it to the display level"))
+      warning("collapsing level cannot be smaller than display level; setting it to the display level")
     }
     if ((dlevel < clevel) || (clevel < Q)) {
       collapseGroups <-
@@ -294,7 +295,7 @@ collapse.groupedData <-
       }
       if (!is.null(preserve)) {
         if (!(inherits(preserve, "formula") && length(preserve) == 2)) {
-          stop("\"Preserve\" must be a two-sided formula")
+          stop("'preserve' must be a two-sided formula")
         }
         collapseGroups <- paste(collapseGroups, eval(preserve[[2]], object),
                                 sep = "\007")
@@ -339,7 +340,7 @@ collapse.groupedData <-
     }
     displayGroups <- ordered(displayGroups,
       levels = unique(as.character(displayGroups[do.call("order", groups)])))
-    form[[3]][[3]] <- as.name(".groups")
+    form[[3]][[3]] <- quote(.groups)
     object[[".groups"]] <- displayGroups
     args[["formula"]] <- form
     args[["data"]] <- object
@@ -381,7 +382,7 @@ plot.nfnGroupedData <-
            }, key = TRUE, grid = TRUE, ...)
 {
   labels <- list(xlab = xlab, ylab =  ylab)
-  labels <- labels[unlist(lapply(labels, length)) > 0]
+  labels <- labels[lengths(labels) > 0]
   args <- c(list(attr(x, "formula"), data = x, strip = strip,
 		 aspect = aspect, panel = panel), labels)
   if (length(outer) > 0) {
@@ -483,7 +484,7 @@ plot.nffGroupedData <-
     groupLabel <- groupExpr
   }
   labels <- list(xlab = xlab, ylab = ylab)
-  labels <- labels[unlist(lapply(labels, length)) > 0]
+  labels <- labels[lengths(labels) > 0]
   if (length(outer) > 0) {
     if (is.logical(outer) && outer) {	# get the default outer formula
       form <- formula(paste(groupExpr,
@@ -559,11 +560,12 @@ plot.nmGroupedData <-
   do.call("plot", args)
 }
 
-print.groupedData <-
-  function(x, ...)
+print.groupedData <- function(x, ...)
 {
   cat("Grouped Data: ")
-  print(attr(x, "formula"))
+  if(identical(emptyenv(), environment(frm <- attr(x, "formula"))))
+      environment(frm) <- globalenv()# for printing, as that will be suppressed
+  print(frm, ...)
   print.data.frame(x, ...)
 }
 
@@ -626,11 +628,11 @@ asTable.groupedData <-
   function(object)
 {
   if (length(getGroupsFormula(object, asList = TRUE)) > 1) {
-    stop("asTable cannot be used with multilevel grouped data")
+    stop("'asTable' cannot be used with multilevel grouped data")
   }
   tab <- table( getGroups(object), getCovariate(object) )
   if (1 != length(unique(tab)))
-    stop("asTable can only be used with balanced groupedData objects")
+    stop("'asTable' can only be used with balanced 'groupedData' objects")
   tab[] <- getResponse(object)[order(getCovariate(object),getGroups(object))]
   tab
 }
@@ -650,9 +652,3 @@ balancedGrouped <-
   frm[[ as.character(getResponseFormula(form)[[2]]) ]] <- as.vector( data )
   do.call("groupedData", list(form, data = frm, labels = labels, units = units ))
 }
-
-### Local variables:
-### mode: S
-### End:
-
-
