@@ -1,8 +1,8 @@
 ###              Classes of positive-definite matrices
 ###
+### Copyright 2006-2017  The R Core team
 ### Copyright 1997-2003  Jose C. Pinheiro,
 ###                      Douglas M. Bates <bates@stat.wisc.edu>
-# Copyright 2006-2012 The R Core team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -179,7 +179,7 @@ pdConstruct.pdMat <-
         if (any(noMatch <- is.na(match(nam, namesForm)))) {
           err <- TRUE
           namCopy <- nam
-          indNoMatch <- (1:length(nam))[noMatch]
+          indNoMatch <- seq_along(nam)[noMatch]
           if (any(wch1 <- (nchar(nam, "c") > 12))) {
             ## possibly names with .(Intercept) in value
             wch1 <- substring(nam, nchar(nam, "c")-10) == "(Intercept)"
@@ -265,37 +265,29 @@ coef.pdMat <-
   value
 }
 
-Dim.pdMat <-
-  function(object, ...)
+Dim.pdMat <- function(object, ...)
 {
-  if ((val <- length(Names(object))) > 0) {
-    return(c(val, val))
-  } else if (isInitialized(object)) {
-    return(dim(as.matrix(object)))
-  }
-  stop("cannot access the number of columns of uninitialized objects without names")
+  if ((val <- length(Names(object))) > 0)
+    c(val, val)
+  else if (isInitialized(object))
+    dim(as.matrix(object))
+  else
+    stop("cannot access the number of columns of uninitialized objects without names")
 }
 
 formula.pdMat <-
   function(x, asList, ...) eval(attr(x, "formula"))
 
-isInitialized.pdMat <-
-  function(object)
-{
-  length(object) > 0
-}
+isInitialized.pdMat <- function(object) length(object) > 0
 
-logDet.pdMat <-
-  function(object, ...)
+logDet.pdMat <- function(object, ...)
 {
-  if (!isInitialized(object)) {
+  if (!isInitialized(object))
     stop("cannot extract the log of the determinant from an uninitialized object")
-  }
-  sum(log(svd(pdMatrix(object, factor = TRUE))$d))
+  sum(log(svd.d(pdMatrix(object, factor = TRUE))))
 }
 
-"matrix<-.pdMat" <-
-  function(object, value)
+`matrix<-.pdMat` <- function(object, value)
 {
   value <- as.matrix(value)
   ## check for consistency of dimensions when object is initialized
@@ -311,8 +303,7 @@ Names.pdMat <-
   as.character(attr(object, "Dimnames")[[2]])
 }
 
-"Names<-.pdMat" <-
-  function(object, ..., value)
+`Names<-.pdMat` <- function(object, ..., value)
 {
   if (is.null(value)) {
     attr(object, "Dimnames") <- NULL
@@ -336,7 +327,7 @@ Names.pdMat <-
       err <- TRUE
       ## checking nlme case
       valueCopy <- value
-      indNoMatch <- (1:length(value))[noMatch]
+      indNoMatch <- seq_along(value)[noMatch]
       nam1 <- value[noMatch]            # no matching names
       if (any(wch1 <- (nchar(nam1, "c") > 12))) {
         ## possibly names with .(Intercept) in value
@@ -425,8 +416,8 @@ plot.pdMat <-
 			    y = sumDif[1,2] + lagged * sumDif[2,2])
 		     }, lagged = laggedCos)
     gg <- rep(seq_along(xylist), rep(length(.angles), length(xylist)))
-    panel.superpose(unlist(lapply(xylist, "[[", "x")),
-		    unlist(lapply(xylist, "[[", "y")),
+    panel.superpose(unlist(lapply(xylist, `[[`, "x")),
+		    unlist(lapply(xylist, `[[`, "y")),
 		    subscripts = seq_along(gg), groups = gg, ..., type = "l")
   }, subscripts = TRUE, groups = groups)
 }
@@ -629,16 +620,14 @@ pdFactor.pdSymm <-
      as.double(object))$Factor
 }
 
-pdMatrix.pdSymm <-
-  function(object, factor = FALSE)
+pdMatrix.pdSymm <- function(object, factor = FALSE)
 {
-  if (!isInitialized(object)) {
+  if (!isInitialized(object))
     stop("cannot extract matrix from an uninitialized object")
-  }
   if (factor) {
     Ncol <- Dim(object)[2]
     value <- array(pdFactor(object), c(Ncol, Ncol), attr(object, "Dimnames"))
-    attr(value, "logDet") <- sum(log(abs(svd(value)$d)))
+    attr(value, "logDet") <- sum(log(abs(svd.d(value))))
     value
   } else {
     NextMethod()
@@ -763,7 +752,7 @@ solve.pdLogChol <-
   if (!isInitialized(a)) {
     stop("cannot get the inverse of an uninitialized object")
   }
-  Ncol <- (-1 + sqrt(1 + 8 * length(a))) / 2
+#  Ncol <- (-1 + sqrt(1 + 8 * length(a))) / 2
 #  val <- array(.Fortran("dbksl",
 # 			as.double(pdFactor(a)),
 # 			as.integer(Ncol),
@@ -1872,7 +1861,7 @@ pdConstruct.pdBlocked <-
     stop("all elements in the argument must generate \"pdMat\" objects")
   }
   namesList <- lapply(object, Names)
-  lNam <- unlist(lapply(namesList, length))
+  lNam <- lengths(namesList)
 #  namInterc <- unlist(lapply(namesList,
 #                             function(el) {
 #                               (length(el) == 1) && (el == "(Intercept)")
@@ -2123,27 +2112,19 @@ summary.pdBlocked <-
     val <- eval(mCall)
     vNames <- colnames(val)
     auxNames <- lapply(Names(xx, TRUE),
-		       function(el, vN) {
-			 aux <- match(vN, el)
-			 if (any(aux1 <- !is.na(aux))) {
-			   el[aux[aux1]]
-			 }
-		       }, vN = vNames)
-    auxWhich <- !unlist(lapply(auxNames, is.null))
+		       function(el) {
+			 aux <- match(vNames, el)
+			 if(any(ok <- !is.na(aux))) el[aux[ok]] # else NULL
+		       })
+    auxWhich <- !vapply(auxNames, is.null, NA)
     if (sum(auxWhich) == 1) {
-      return(pdConstruct(as.list(xx)[auxWhich][[1]], val))
+	pdConstruct(as.list(xx)[auxWhich][[1]], val)
+    } else {
+	auxNames <- auxNames[auxWhich]
+	auxClass <- vapply(xx, function(el) class(el)[1L], "")[auxWhich]
+	pdConstruct(xx, val, nam = auxNames, form = NULL, pdClass = auxClass)
     }
-    auxNames <- auxNames[auxWhich]
-    auxClass <- unlist(lapply(xx, function(el) class(el)[1]))[auxWhich]
-    return(pdConstruct(xx, val, nam = auxNames, form = NULL,
-		       pdClass = auxClass))
   } else {
     eval(mCall)
   }
 }
-
-### Local variables:
-### mode: S
-### End:
-
-
