@@ -1,7 +1,7 @@
 #  File src/library/base/R/library.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2017 The R Core Team
+#  Copyright (C) 1995-2018 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -105,28 +105,6 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
                  call. = FALSE, domain = NA)
     }
 
-    testFeatures <- function(features, pkgInfo, pkgname, pkgpath)
-    {
-        ## Check that the internals version used to build this package
-        ## matches the version of current R. Failure in this test
-        ## should only occur if the R version is an unreleased devel
-        ## version or the package was build with an unrelease devel
-        ## version.  Other mismatches should be caught earlier by the
-        ## version checks.
-        needsComp <- as.character(pkgInfo$DESCRIPTION["NeedsCompilation"])
-        if (identical(needsComp, "yes")) {
-            internalsID <- features$internalsID
-            if (is.null(internalsID))
-                ## the initial internalsID for packages installed
-                ## prior to introducing features.rds in the meta data
-                internalsID <- "0310d4b8-ccb1-4bb8-ba94-d36a55f60262"
-            if (internalsID != .Internal(internalsID()))
-                stop(gettextf("package %s was installed by an R version with different internals; it needs to be reinstalled for use with this R version",
-                              sQuote(pkgname)), call. = FALSE, domain = NA)
-        }
-    }
-
-
     checkNoGenerics <- function(env, pkg)
     {
         nenv <- env
@@ -251,11 +229,8 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
                               sQuote(package)), domain = NA)
             pkgInfo <- readRDS(pfile)
             testRversion(pkgInfo, package, pkgpath)
-            ffile <- system.file("Meta", "features.rds", package = package,
-                                 lib.loc = which.lib.loc)
-            features <- if (file.exists(ffile)) readRDS(ffile) else NULL
-            testFeatures(features, pkgInfo, package, pkgpath)
 
+            ## The ABI compatibility check is now in loadNamespace
             ## The licence check is now in loadNamespace
             ## The check for inconsistent naming is now in find.package
 
@@ -280,15 +255,15 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
                     oldversion <- as.numeric_version(getNamespaceVersion(package))
                     if (newversion != oldversion) {
                     	## No, so try to unload the previous one
-                    	res <- tryCatch(unloadNamespace(package),
-					error = function(e) {
-					    P <- if(!is.null(cc <- conditionCall(e)))
-						     paste("Error in", deparse(cc)[1L], ": ")
-						 else "Error : "
-					    stop(gettextf("Package %s version %s cannot be unloaded:\n %s",
-							  sQuote(package), oldversion,
-							  paste0(P, conditionMessage(e),"\n")),
-						 domain=NA)})
+			tryCatch(unloadNamespace(package),
+				 error = function(e) {
+				     P <- if(!is.null(cc <- conditionCall(e)))
+					      paste("Error in", deparse(cc)[1L], ": ")
+					  else "Error : "
+				     stop(gettextf("Package %s version %s cannot be unloaded:\n %s",
+						   sQuote(package), oldversion,
+						   paste0(P, conditionMessage(e),"\n")),
+					  domain=NA)})
                     }
                 }
 		tt <- tryCatch({
@@ -736,10 +711,12 @@ function(package = NULL, lib.loc = NULL, quiet = FALSE,
         }
         if(length(paths) > 1L) {
             ## If a package was found more than once ...
-            paths <- paths[1L]
 	    if(verbose)
-                warning(gettextf("package %s found more than once,\nusing the one found in %s",
-                                 sQuote(pkg), sQuote(paths)), domain = NA)
+		warning(gettextf("package %s found more than once, using the first from\n  %s",
+				 sQuote(pkg),
+				 paste(dQuote(paths), collapse=",\n  ")),
+			domain = NA)
+            paths <- paths[1L]
         }
         out <- c(out, paths)
     }
